// Sébastien Clara - Avril 2013 - https://sebastienclara.wordpress.com/
// A dynamic sampler instrument on SuperCollider
/////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////
// synthétiseurs
/////////////////////////////////////////////////////////////////////////////////
(
SynthDef(\loopBufferStereo, { arg out=0, buffer, rate=1, amp=0.7, position=0, release=3, gate=1;
	var signal, env;
	env =  EnvGen.kr(Env.asr(attackTime:0.01, sustainLevel:1, releaseTime:release, curve:-1.5), gate, doneAction:2);

	signal =  PlayBuf.ar(1, buffer, BufRateScale.kr(buffer)*rate, loop:1);
	signal =  Pan2.ar(signal, position); //Pan position, -1 is left, +1 is right.
    Out.ar(out, signal * env * amp)
}).add;
//////////////////////////////
SynthDef(\reverb, { arg out=0, in, gate=1, mix=0.33, room=0.5, damp=0.5;
	var signal, env;
	env =  EnvGen.kr(Env.asr(attackTime:0.01, releaseTime:0.01, curve:\welch), gate, doneAction:2);

	signal = FreeVerb.ar(In.ar(in, 2), mix, room, damp);
	Out.ar(out, signal * env );
}).add;

SynthDef(\delay, { arg out=0, in, gate=1, maxdelaytime=0.2, delaytime=0.2, decaytime=1;
	var signal, env;
	env =  EnvGen.kr(Env.asr(attackTime:0.01, releaseTime:0.01, curve:\welch), gate, doneAction:2);

	signal = CombC.ar(In.ar(in, 2), maxdelaytime, delaytime, decaytime);
	Out.ar(out, signal * env );
}).add;
//////////////////////////////
SynthDef(\filter, { arg out=0, in, gate=1, freq, q, mode, saturation;
	var signal, env;
	env =  EnvGen.kr(Env.asr(attackTime:0.01, releaseTime:0.01, curve:\welch), gate, doneAction:2);

	signal = BMoog.ar(In.ar(in, 2), In.kr(freq), In.kr(q), mode, saturation );
	ReplaceOut.ar(out, signal * env);
}).add;

SynthDef(\autoControlFilter, { arg out1, out2, speed=3;
	Out.kr(out1, LFNoise1.kr(speed).range(250,10000) );
	Out.kr(out2, LFNoise1.kr(speed).range(0,0.8) );
}).add;

SynthDef(\fixedControlFilter, { arg out1, out2, freq=400, q=0.1;
	Out.kr(out1, freq);
	Out.kr(out2, q);
}).add;
//////////////////////////////
SynthDef(\mainVol, { arg out=0, in, amp=1;
	Out.ar(out, In.ar(in, 2) * amp);
}).add;
)

/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
// guiApp
/////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////
(
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
// initialisation
var fenetre, bufferAudio, instrumentGroup, busAudio, busControlFreq, busControlQ;
var mainVolumeSynth, instruReverb, instruDelay, instruFilter;
var instruAutoFilter, instruFixedFilter;
var allocationBuffTxt, allocTxt, instrumentTxt, mainVolumeTxt, mainReleaseTxt;
var allocNum, mainReleaseNum;
var mainVolumeSlider;
var recButton, addInstrumentButton, removeInstrumentButton, saveButton, startAllButton, stopAllButton;
var samplePop;
var saveBufferFiel;
var panControl;
var fctSaveBuffer, doYouRec=false, numberInstrument=0, maxInstrument=8;
var samplerInstrumentTab = Array.fill(maxInstrument,{nil}), dynamiqueInstrumentLayout = VLayout();
var guiInstrumentTab = Array2D.new(maxInstrument,10);

var reverbButton, reverbMixTxt, reverbRoomTxt, reverbDampTxt, reverbMixNum, reverbRoomNum, reverbDampNum;
var delayButton, maxDelayTxt, delayTxt, decayTxt, maxDelayNum, delayNum, decayNum;
var filterButton, filterPopUp, filterFreqTxt, filterQTxt, filterFreqNum, filterQNum, filterSatTxt, filterSatNum;
var filterAutoButton, filterAutoTxt, filterAutoNum;
var objetsVisible;

var pathApp = PathName(thisProcess.nowExecutingPath).pathOnly;
var tabSoundFiles = ( PathName(pathApp ++ "sample") ).files;

instrumentGroup = Group.new;
busAudio = Bus.audio(s, 2);
busControlFreq = Bus.control(s, 1);
busControlQ = Bus.control(s, 1);
mainVolumeSynth = Synth(\mainVol, [\out,0, \in,busAudio], instrumentGroup, addAction:\addToTail);
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
// Window
//fenetre = Window("MultiSampler");
fenetre = Window("MultiSampler", Rect(330, 50, 600, 800));

allocationBuffTxt = StaticText(fenetre).align_(\center).string_("Time buffer");
allocNum = NumberBox(fenetre).align_(\center).clipLo_(1).clipHi_(15).step_(1).scroll_step_(1).value_(5);
allocTxt = StaticText(fenetre).align_(\center).font_(Font("Monaco", 12, true));
recButton = Button(fenetre).states_([["Rec", Color.black, Color.gray(0.9,1) ]]);
saveButton = Button(fenetre).states_([["Save"]]);
saveBufferFiel = TextField(fenetre).string_("sample");
samplePop = PopUpMenu(fenetre);
tabSoundFiles.do({|f| samplePop.items = samplePop.items.add(f.fileNameWithoutExtension) });

instrumentTxt = StaticText(fenetre).align_(\center).string_("Add or remove a sampler");
addInstrumentButton = Button(fenetre).states_([["Add"]]);
removeInstrumentButton = Button(fenetre).states_([["Remove"]]);
startAllButton = Button(fenetre).states_([["Start all"]]);
stopAllButton = Button(fenetre).states_([["Stop all"]]);

reverbButton = Button(fenetre).states_([["Reverb", Color.black, Color.gray(0.9,1)],["Reverb", Color.black, Color.green(0.9,1) ]]);
reverbMixTxt = StaticText(fenetre).align_(\center).string_("Mix");
reverbRoomTxt = StaticText(fenetre).align_(\center).string_("Room");
reverbDampTxt = StaticText(fenetre).align_(\center).string_("Damp");
reverbMixNum = NumberBox(fenetre).align_(\center).clipLo_(0).clipHi_(1).step_(0.1).scroll_step_(0.1).value_(0);
reverbRoomNum = NumberBox(fenetre).align_(\center).clipLo_(0).clipHi_(1).step_(0.1).scroll_step_(0.1).value_(0.3);
reverbDampNum = NumberBox(fenetre).align_(\center).clipLo_(0).clipHi_(1).step_(0.1).scroll_step_(0.1).value_(1);
delayButton = Button(fenetre).states_([["Delay", Color.black, Color.gray(0.9,1)],["Delay", Color.black, Color.magenta(0.9,1) ]]);
maxDelayTxt = StaticText(fenetre).align_(\center).string_("Max Delay");
delayTxt = StaticText(fenetre).align_(\center).string_("Delay");
decayTxt = StaticText(fenetre).align_(\center).string_("Decay");
maxDelayNum = NumberBox(fenetre).align_(\center).clipLo_(0).clipHi_(5).step_(1).scroll_step_(1).value_(3);
delayNum = NumberBox(fenetre).align_(\center).clipLo_(0).clipHi_(3).step_(0.5).scroll_step_(0.5).value_(0.5);
decayNum = NumberBox(fenetre).align_(\center).clipLo_(0).clipHi_(10).step_(1).scroll_step_(1).value_(5);

filterButton = Button(fenetre).states_([["Filter", Color.black, Color.gray(0.9,1)],["Filter", Color.black, Color.blue(0.9,0.6) ]]);
filterPopUp = PopUpMenu(fenetre).items_(["low pass", "high pass", "bandpass"]);
filterFreqTxt = StaticText(fenetre).align_(\center).string_("Frequency");
filterFreqNum = NumberBox(fenetre).align_(\center).clipLo_(100).clipHi_(15000).step_(100).scroll_step_(100).value_(400);
filterQTxt = StaticText(fenetre).align_(\center).string_("Q");
filterQNum = NumberBox(fenetre).align_(\center).clipLo_(0).clipHi_(1).step_(0.05).scroll_step_(0.05).value_(0.1);
filterSatTxt = StaticText(fenetre).align_(\center).string_("Saturation");
filterSatNum = NumberBox(fenetre).align_(\center).clipLo_(0).clipHi_(1).step_(0.05).scroll_step_(0.05).value_(0.5);
filterAutoButton = Button(fenetre).states_([["Auto", Color.black, Color.gray(0.9,1)],["Auto", Color.black, Color.red(0.9,0.5) ]]);
filterAutoTxt = StaticText(fenetre).align_(\center).string_("Speed");
filterAutoNum = NumberBox(fenetre).align_(\center).clipLo_(0.1).clipHi_(15).step_(0.1).scroll_step_(0.1).value_(1);

mainVolumeTxt = StaticText(fenetre).align_(\center).string_("Main volume");
mainVolumeSlider = Slider(fenetre).orientation_(\horizontal).value_(1);
mainReleaseTxt = StaticText(fenetre).align_(\center).string_("Main release");
mainReleaseNum = NumberBox(fenetre).align_(\center).clipLo_(0.5).clipHi_(10).step_(0.5).scroll_step_(0.5).value_(1);

fenetre.layout = HLayout( VLayout(
	HLayout(instrumentTxt, addInstrumentButton, removeInstrumentButton),

	HLayout(allocationBuffTxt, allocNum, allocTxt, recButton ),
	HLayout(samplePop, saveBufferFiel, saveButton),

	HLayout(delayButton,
		VLayout(maxDelayTxt, maxDelayNum),
		VLayout(delayTxt, delayNum),
		VLayout(decayTxt, decayNum)),

	HLayout(
		VLayout(filterButton, filterPopUp),
		VLayout(filterFreqTxt, filterFreqNum),
		VLayout(filterQTxt, filterQNum),
		VLayout(filterSatTxt, filterSatNum),
		filterAutoButton,
		VLayout(filterAutoTxt, filterAutoNum) ),

	HLayout(reverbButton,
		VLayout(reverbMixTxt, reverbMixNum),
		VLayout(reverbRoomTxt, reverbRoomNum),
		VLayout(reverbDampTxt, reverbDampNum)),

	HLayout([VLayout(mainVolumeTxt, mainVolumeSlider), stretch:4],
		[VLayout([mainReleaseTxt, align:\center], [mainReleaseNum, align:\center]), stretch:1] ),

	HLayout(startAllButton, stopAllButton),
	dynamiqueInstrumentLayout
));
fenetre.front;
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
// Action
recButton.action = {
	if(doYouRec.not, {
		doYouRec=true;
		if( allocTxt.string.asInteger != allocNum.value, {
			//maj du buffer
			allocTxt.string = allocNum.value.asString;
			bufferAudio.free;
			bufferAudio = Buffer.alloc(s, s.sampleRate * allocNum.value, 1);
		});
		recButton.states = [["Rec", Color.black, Color.red]];
		{	var in;
			in = AudioIn.ar(1) * EnvGen.kr(Env([0,1,1,0],[1,98,1].normalizeSum,\welch), timeScale:BufDur.kr(bufferAudio));
			RecordBuf.ar(in, bufferAudio, doneAction: 2, loop: 0);
			0.0 //quiet
		}.play;
		{	recButton.states = [["Rec", Color.black, Color.gray(0.9,1) ]];
			doYouRec=false;
		}.defer(allocTxt.string.asInteger);
	});
};
fctSaveBuffer = {
	samplePop.items = samplePop.items.add(saveBufferFiel.string);
	samplePop.value	 = samplePop.items.size-1;
	bufferAudio.write( (pathApp ++ "sample/" ++ saveBufferFiel.string ++ ".wav").standardizePath, "wav", "int16", -1, 0, true);
	("Save" + saveBufferFiel.string ++ ".wav on hard disk.\n" ).postln;
};
saveButton.action = {
	if( samplePop.items.notNil, {
		if( samplePop.items.find([saveBufferFiel.string]).notNil, {
			(saveBufferFiel.string + "exist !!!\nChange the name in the text field.\n" ).postln;
		},{
			fctSaveBuffer.value;
		});
	},{
		fctSaveBuffer.value;
	});
};
samplePop.action = {
	bufferAudio.free;
	bufferAudio = Buffer.read(s, (pathApp ++ "sample/" ++ samplePop.item.asString ++ ".wav").standardizePath );
};
if(samplePop.items.notNil,{samplePop.action.value;});

/////////////////////
// effets, contrôles
mainVolumeSlider.action = { arg slid;
	mainVolumeSynth.set(\amp,slid.value);
};
mainReleaseNum.action = { arg num;
	numberInstrument.do({ arg index;
		guiInstrumentTab[index, 4].value = num.value;
	});
};
reverbButton.action = { arg bouton;
	if(bouton.value==1, {
		instruReverb = Synth.before (mainVolumeSynth, \reverb,
			[\out,busAudio, \in,busAudio,
				\mix, reverbMixNum.value,
				\room, reverbRoomNum.value,
				\damp, reverbDampNum.value]);
	},{
			instruReverb.set(\gate, 0);
	});
};
reverbMixNum.action = { arg num;
	if(reverbButton.value==1, {
		instruReverb.set(\mix, num.value);
	});
};
reverbRoomNum.action = { arg num;
	if(reverbButton.value==1, {
		instruReverb.set(\room, num.value);
	});
};
reverbDampNum.action = { arg num;
	if(reverbButton.value==1, {
		instruReverb.set(\damp, num.value);
	});
};
delayButton.action = { arg bouton;
	if(bouton.value==1, {
		instruDelay = Synth.before (mainVolumeSynth, \delay,
			[\out,busAudio, \in,busAudio,
				\maxdelaytime, maxDelayNum.value,
				\delaytime, delayNum.value,
				\decaytime, decayNum.value]);
	},{
			instruDelay.set(\gate, 0);
	});
};
maxDelayNum.action = { arg num;
	if(delayButton.value==1, {
		instruDelay.set(\maxdelaytime, num.value);
	});
};
delayNum.action = { arg num;
	if(delayButton.value==1, {
		instruDelay.set(\delaytime, num.value);
	});
};
decayNum.action = { arg num;
	if(delayButton.value==1, {
		instruDelay.set(\decaytime, num.value);
	});
};


filterButton.action = { arg bouton;
	if(bouton.value==1, {
		if(filterAutoButton.value == 0, {
			instruFixedFilter = Synth(\fixedControlFilter,
				[\out1, busControlFreq,
					\out2, busControlQ,
					\freq, filterFreqNum.value,
					\q, filterQNum.value],
				instrumentGroup, addAction:\addToHead );
		}, {
			objetsVisible.value(false);
			instruAutoFilter = Synth(\autoControlFilter,
				[\out1, busControlFreq,
					\out2, busControlQ,
					\speed, filterAutoNum.value ],
				instrumentGroup, addAction:\addToHead );
		});

		instruFilter = Synth.before (mainVolumeSynth, \filter,
			[\out,busAudio, \in,busAudio,
				\freq,busControlFreq, \q,busControlQ,
				\saturation,filterSatNum.value,
				\mode,filterPopUp.value ] );
	}, {
		instruFilter.set(\gate, 0);
		if(filterAutoButton.value == 0, {
			instruFixedFilter.free;
		}, {
			objetsVisible.value(true);
			instruAutoFilter.free;
		});
	});
};
objetsVisible = { arg visible=true;
	filterFreqNum.visible=visible; filterQNum.visible=visible;
	filterFreqTxt.visible=visible; filterQTxt.visible=visible;
};
filterFreqNum.action = { arg num;
	if(filterButton.value==1, {
		instruFixedFilter.set(\freq, num.value );
	});
};
filterQNum.action = { arg num;
	if(filterButton.value==1, {
		instruFixedFilter.set(\q, num.value );
	});
};
filterSatNum.action = { arg num;
	if(filterButton.value==1, {
		instruFilter.set(\saturation, num.value );
	});
};
filterPopUp.action = { arg menu;
	if(filterButton.value==1, {
		instruFilter.set(\mode, menu.value );
	});
};
filterPopUp.value_(0);

filterAutoButton.action = { arg bouton;
	if(filterButton.value==1, {
		if(bouton.value == 0, {
			objetsVisible.value(true);

			instruFixedFilter = Synth(\fixedControlFilter,
					[\out1, busControlFreq,
						\out2, busControlQ,
						\freq, filterFreqNum.value,
						\q, filterQNum.value ],
					instrumentGroup, addAction:\addToHead );
			instruAutoFilter.free;
		},{
			objetsVisible.value(false);

			instruAutoFilter = Synth(\autoControlFilter,
				[\out1, busControlFreq,
					\out2, busControlQ,
					\speed, filterAutoNum.value ],
				instrumentGroup, addAction:\addToHead );
			instruFixedFilter.free;
		});
	});
};
filterAutoNum.action = { arg num;
	if(filterButton.value==1, {
		instruAutoFilter.set(\speed, filterAutoNum.value);
	});
};
/////////////////////
// multiSampler
addInstrumentButton.action = {
	var numInstru = numberInstrument;
	if( numberInstrument!=maxInstrument, {
		numberInstrument = numberInstrument+1;

		guiInstrumentTab[numInstru, 0] =
		Button(fenetre).states_([["Play", Color.black, Color.gray(0.9,1)],["Stop", Color.black, Color.cyan(0.9,1) ]]).action_({ arg bouton;
			var sense = if(guiInstrumentTab[numInstru, 9].value == 0, {1},{-1});
			if(bouton.value==1, {
				samplerInstrumentTab.put(numInstru,
					Synth(\loopBufferStereo, [ \out,busAudio,
						\buffer,bufferAudio,
						\rate,guiInstrumentTab[numInstru, 2].value.midiratio * sense,
						\release,guiInstrumentTab[numInstru, 4].value,
						\position,panControl.map(guiInstrumentTab[numInstru, 6].value),
						\amp,guiInstrumentTab[numInstru, 8].value],
						instrumentGroup, addAction:\addToHead)
				);
			}, {
				samplerInstrumentTab[numInstru].set(\gate,0,
					\release,guiInstrumentTab[numInstru, 4].value);
			});
		});

		guiInstrumentTab[numInstru, 1] = StaticText(fenetre).align_(\center).string_("Transpose");
		guiInstrumentTab[numInstru, 2] =
		NumberBox(fenetre).align_(\center).clipLo_(-36).clipHi_(36).step_(1).scroll_step_(1).value_(0).action_({ arg num;
			if(guiInstrumentTab[numInstru, 0].value==1, {
				if(guiInstrumentTab[numInstru, 9].value == 0, {
					samplerInstrumentTab[numInstru].set(\rate,num.value.midiratio * 1)
				},{
					samplerInstrumentTab[numInstru].set(\rate,num.value.midiratio * -1)
				});

			});
		});

		guiInstrumentTab[numInstru, 3] = StaticText(fenetre).align_(\center).string_("Release");
		guiInstrumentTab[numInstru, 4] =
		NumberBox(fenetre).align_(\center).clipLo_(0.5).clipHi_(10).step_(0.5).scroll_step_(0.5).value_(mainReleaseNum.value);

		guiInstrumentTab[numInstru, 5] = StaticText(fenetre).align_(\center).string_("Pan");
		guiInstrumentTab[numInstru, 6] =
		Slider(fenetre).orientation_(\horizontal).value_(0.5).action_({ arg slid;
			if(guiInstrumentTab[numInstru, 0].value==1, {
				samplerInstrumentTab[numInstru].set(\position,panControl.map(slid.value))
			});
		});

		guiInstrumentTab[numInstru, 7] = StaticText(fenetre).align_(\center).string_("Vol");
		guiInstrumentTab[numInstru, 8] =
		Slider(fenetre).orientation_(\horizontal).value_(0.5).action_({ arg slid;
			if(guiInstrumentTab[numInstru, 0].value==1, {
				samplerInstrumentTab[numInstru].set(\amp,slid.value)
			});
		});

		guiInstrumentTab[numInstru, 9] = Button(fenetre).states_([["Reverse", Color.black, Color.gray(0.9,1)],["Reverse", Color.black, Color.yellow(0.9,1) ]]).action_({ arg bouton;
			if(guiInstrumentTab[numInstru, 0].value==1, {
				if(bouton.value == 0, {
					samplerInstrumentTab[numInstru].set(\rate,guiInstrumentTab[numInstru, 2].value.midiratio * 1)
				},{
					samplerInstrumentTab[numInstru].set(\rate,guiInstrumentTab[numInstru, 2].value.midiratio * -1)
				});
			});
		});

		dynamiqueInstrumentLayout.add (	VLayout(
			HLayout(VLayout(guiInstrumentTab[numInstru,0],
						guiInstrumentTab[numInstru,9]),
				VLayout(guiInstrumentTab[numInstru,1], guiInstrumentTab[numInstru,2]),
				VLayout(guiInstrumentTab[numInstru,3], guiInstrumentTab[numInstru,4])),
			HLayout(guiInstrumentTab[numInstru,5], guiInstrumentTab[numInstru,6],
				guiInstrumentTab[numInstru,7], guiInstrumentTab[numInstru,8])
		) );
	});
};
addInstrumentButton.action.value;
/////////////////////
// Gestion multiSampler
removeInstrumentButton.action = {
	if(numberInstrument!=1,{
		if(guiInstrumentTab[numberInstrument-1, 0].value==1, {
			samplerInstrumentTab[numberInstrument-1].set(\release,0.01, \gate,0);
		});
		guiInstrumentTab.rowAt(numberInstrument-1).do(_.destroy);
		numberInstrument = numberInstrument-1;
	});
};
startAllButton.action = {
	numberInstrument.do({ arg index;
		if(guiInstrumentTab[index, 0].value==0, {
			guiInstrumentTab[index, 0].valueAction=1;
		});
	});
};
stopAllButton.action = {
	numberInstrument.do({ arg index;
		if(guiInstrumentTab[index, 0].value==1, {
			guiInstrumentTab[index, 0].valueAction=0;
		});
	});
};

fenetre.onClose = {
	numberInstrument.do({ arg index;
		if(guiInstrumentTab[index, 0].value==1, {
			samplerInstrumentTab[index].set(\release,0.01, \gate,0)
		});
	});

	bufferAudio.free;
	busAudio.free; instrumentGroup.free;
	busControlFreq.free; busControlQ.free;
};
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////
// Contrôle
panControl = ControlSpec(minval: -1, maxval: 1, warp:\lin, step: 0.05, default:0);

// Contrôle par clavier d'ordi
fenetre.view.keyDownAction = { arg view, char, modifiers, unicode, keycode;
	switch (unicode,
		27, { stopAllButton.action.value; }, // <esc>
		32, { startAllButton.action.value; } // <space>
	);
};

"Start\n"
)


